/*
 * File: DrawFace3.java
 * --------------------
 * This program creates a GFace object similar to DrawFace2,
 * but also it has the eyes track the mouse.
 */

package demos;

import acm.graphics.*;
import acm.program.*;
import java.awt.event.*;

/**
 * The DrawFace3 class represents the main program, but the
 * real work in this design is left to the GFace object.
 * This version of DrawFace has been extended to with code
 * to track the mouse.
 */

@SuppressWarnings("serial")
public class DrawFace3 extends GraphicsProgram {

	/** Runs the program */
	public void run() {
		face = new GFace3(FACE_WIDTH, FACE_HEIGHT);
		add(face, getWidth() / 2, getHeight() / 2);
		addMouseListeners();
	}
	
	/** Called when the mouse is moved */
	public void mouseMoved(MouseEvent e) {
		face.lookAt(e.getX(), e.getY());
	}

	/* Constants */
	private static final double FACE_WIDTH = 200;
	private static final double FACE_HEIGHT = 300;
	
	/* Private instance variable */
	private GFace3 face;
}

	/**
	 * The GFace class represents an abstract graphical rendering
	 * of a face with geometric figures for the eyes, nose, and mouth.
	 */
class GFace3 extends GCompound {
	
	/** Creates a new instance of the GFace3 class */
	public GFace3(double width, double height) {
		head = new GOval(width, height);	
		leftEye = new GEye3(EYE_WIDTH * width, EYE_HEIGHT * height);
		rightEye = new GEye3(EYE_WIDTH * width, EYE_HEIGHT * height);
		nose = new GNose3(NOSE_WIDTH * width, NOSE_HEIGHT * height);
		mouth = new GMouth3(MOUTH_WIDTH * width, MOUTH_HEIGHT * height);
		
		add(head, -width / 2, -height / 2);
		add(leftEye, -0.25 * width, -0.25 * height);
		add(rightEye, 0.25 * width, -0.25 * height);
		add(nose, 0, 0);
		add(mouth, 0, 0.25 * height);
	}
	
	/**
	 * Shifts the eyes to look at the point (x, y).  At this level,
	 * the code simply passes the message along to each eye.
	 */
	public void lookAt(double x, double y) {
		leftEye.lookAt(x, y);
		rightEye.lookAt(x, y);
	}
	
	/* Parameters */
	private static final double EYE_WIDTH = 0.15;
	private static final double EYE_HEIGHT = 0.15;
	private static final double NOSE_WIDTH = 0.15;
	private static final double NOSE_HEIGHT = 0.10;
	private static final double MOUTH_WIDTH = 0.50;
	private static final double MOUTH_HEIGHT = 0.03;
	
	/* Private instance variables */
	private GOval head;
	private GEye3 leftEye, rightEye;
	private GNose3 nose;
	private GMouth3 mouth;
}

/**
 * The GEye class represents an eye.  This version subclassess
 * GCompound rather than GOval because doing so provides a simple
 * means to shift the origin to the center of the figure.  This
 * version includes a circular pupil and a method for focusing
 * the eye on a point on the screen.
 */
class GEye3 extends GCompound {
	public GEye3 (double width, double height) {
		add(new GOval(width, height), -width / 2, - height / 2);
		r = PUPIL_RADIUS_FRACTION * Math.min(width, height);
		pupil = new GOval(2 * r, 2 * r);
		pupil.setFilled(true);
		add(pupil, -r, -r);
	}
	
	/**
	 * Shifts the eye to look at the point (x, y).  Note that
	 * x and y are in the coordinate system of the canvas and
	 * not in the local coordinate system of the compound.  The
	 * call to getCanvasPoint returns the coordinates of the eye
	 * shifted to the space of the canvas.
	 */
	public void lookAt(double x, double y) {
		GPoint pt = getCanvasPoint(0,0);
		double theta = GMath.angle(x - pt.getX(), y - pt.getY());
		pupil.setLocation(-r, -r);
		pupil.movePolar(r, theta);
	}
	
	/** Pupil radius as a function of the eye size */
	private static final double PUPIL_RADIUS_FRACTION =  0.2;
	
	/** Private instance variables */
	private GOval pupil;
	private double r;
}

/**
 * The GNose class represents the nose as a triangular polygon
 */
class GNose3 extends GPolygon {
	public GNose3(double width, double height) {
		addVertex(0, -height / 2);
		addVertex(width / 2, height / 2);
		addVertex(-width / 2, height / 2);
		}
}

/**
 * The GMouth class represents the mouth as a rectangle.  This
 * version subclasses GCompound because doing so provides a simple
 * means to shift the origin to the center of the figure.
 */
class GMouth3 extends GCompound {
	public GMouth3 (double width, double height) {
		add(new GRect(width, height), -width / 2, - height / 2);
	}
}
